function [opt_global, opt_local,optim_set,sims_set,boot_set]=default_settings(K,normalize,no_moms,optim_set,sims_set,boot_set,years)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Starting values and bounds

    if nargin<4
        optim_set=struct();        % Set a blank structure
    end
    if nargin<5
        sims_set=struct();
    end
    if nargin<6
        boot_set=struct();
    end

    % Set to defaults
    if ~isfield(optim_set,'theta_start')
        optim_set.theta_start=ones(K+2,1);
        optim_set.theta_start(end-1:end)=[0;0];
    end
    if isfield(optim_set,'lb') && isfield(optim_set,'ub')
        opt_global.lower_bounds=optim_set.lb;
        opt_global.lower_bounds=optim_set.ub;
    else
        % Constrain/normalizations
        opt_global.lower_bounds=-40*ones(size(optim_set.theta_start));
        opt_global.upper_bounds=40*ones(size(optim_set.theta_start));
        opt_global.lower_bounds(normalize)=0;

        opt_global.lower_bounds(end-1:end)=[0;0];
        opt_global.upper_bounds(end-1:end)=[0;0];
    end

    optim_set.theta_const=(opt_global.lower_bounds==opt_global.upper_bounds);
    assert(sum(optim_set.theta_const)==2);
    assert(all(all([opt_global.lower_bounds<=optim_set.theta_start opt_global.upper_bounds>=optim_set.theta_start])));

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Glocal Optimization defaults - first if using NLOPT
    if ~isfield(optim_set,'global_iter')
        optim_set.global_iter=1000;
    end
    opt_global.maxeval=optim_set.global_iter;
    
    if ~isfield(optim_set,'global_alg')
         optim_set.global_alg=NLOPT_G_MLSL_LDS;
    end
    opt_global.algorithm=optim_set.global_alg;

    if ~isfield(optim_set,'verbose')
         optim_set.verbose=1;
    end
    opt_global.verbose=optim_set.verbose;
    opt_local.verbose=optim_set.verbose;

    if ~isfield(optim_set,'global_local_alg')
        optim_set.global_local_alg=NLOPT_LN_SBPLX;
    end
    opt_global.local_optimizer.algorithm= optim_set.global_local_alg;

    if ~isfield(optim_set,'global_local_xtol')
        optim_set.global_local_xtol=1e-1;
    end
    opt_global.local_optimizer.xtol_rel= optim_set.global_local_xtol;

    if ~isfield(optim_set,'global_local_ftol_rel')
        optim_set.global_local_ftol_rel=1e-2;
    end
    opt_global.local_optimizer.ftol_rel= optim_set.global_local_ftol_rel;

    if ~isfield(optim_set,'global_local_ftol_abs')
        optim_set.global_local_ftol_abs=1e-2;
    end
    opt_global.local_optimizer.ftol_abs=optim_set.global_local_ftol_abs;
    
    %%% Glocal Optimization defaults - only make changes to matlab defaults if using GA
    if ~isfield(optim_set,'ga_UseParallel')
        optim_set.ga_UseParallel='always';
    end
    
    if ~isfield(optim_set,'ga_TolFun')
        optim_set.ga_TolFun=1e-10;
    end

    if ~isfield(optim_set,'ga_Generations')
        optim_set.ga_Generations=200;
    end
    
    % Local Optimization defaults
    if ~isfield(optim_set,'local_alg')
        optim_set.local_alg=NLOPT_LN_SBPLX;
    end
    opt_local.algorithm=optim_set.local_alg;

    if ~isfield(optim_set,'local_xtol_rel')
        optim_set.local_xtol_rel=1e-6;
    end
    opt_local.xtol_rel=optim_set.local_xtol_rel;

    if ~isfield(optim_set,'local_ftol_rel')
        optim_set.local_ftol_rel=1e-10;
    end
    opt_local.ftol_rel=optim_set.local_ftol_rel;

    if ~isfield(optim_set,'local_ftol_abs')
        optim_set.local_ftol_abs=1e-10;
    end
    opt_local.ftol_abs=optim_set.local_ftol_abs;

    if ~isfield(optim_set,'der_tol')
        optim_set.der_tol=eps^(1/4);
    end

    if ~isfield(optim_set,'third_step')
        optim_set.third_step=false;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Set Weight matrix to default
    if ~isfield(optim_set,'W')
        if nargin<7
            years=1;
        end
        optim_set.W=eye(no_moms*years);
    end
    
    % Simulation and bootstrap defaults
    if ~isfield(sims_set,'global_sims')
        sims_set.global_sims=24;
    end
    if ~isfield(sims_set,'sims')
        sims_set.sims=100;
    end
    if ~isfield(sims_set,'der_sims')
        sims_set.der_sims=10000;
    end
    if ~isfield(boot_set,'var_boot')
        boot_set.var_boot=1000;
    end
    if ~isfield(sims_set,'rng_seed')
        sims_set.rng_seed=1;
    end

    if ~isfield(boot_set,'boot_est')
        boot_set.boot_est=false;
    end
    if ~isfield(boot_set,'boot_est_iter')
        boot_set.boot_est_iter=1000;
    end
   
    if ~isfield(boot_set,'sub_W')
        boot_set.sub_W=true;
    end
    if ~isfield(boot_set,'sub_W_iter')
        boot_set.sub_W_iter=1000;
    end   
    
    
end