    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Normalize
    [is_mem,normalize]=ismember(model.normalize,model.u_X);
    if ~is_mem
        error('Could not find normalizing variable');
    end
           
    % Inititialize the set of moments and calculate the moments in
    % the data
    mom_data=mom_spec(mom_specs,data);

    mom_data.moments=match_moms_c(data.match,data.N,data.J,mom_data.X_moms,mom_data.Z_moms,mom_data.XZ_moms,mom_data.mom_ov,mom_data.mom_xz,mom_data.mom_w,mom_data.mom_peer);

    % Clear data as it takes up needless memory - ultimately, call
    % a file to read this
    clear data;


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get default values 
    
    [opt_global,opt_local,optim_set,sims_set,boot_set]=default_settings(K.K,normalize,size(mom_data.moments,1),optim_set,sims_set,boot_set);

    opt_local.lower_bounds=opt_global.lower_bounds/5;
    opt_local.upper_bounds=opt_global.upper_bounds/5;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    output=struct();

    % Simulations, drawn once 
    rng(sims_set.rng_seed);
    sim_data=gen_sims(str_data.N,str_data.J,sims_set.sims,K.rc);
    opt_local.min_objective=@(theta) rc_msm_obj(theta,str_data,sim_data,K,mom_data,optim_set.W);

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Local Optimization
    disp('Running Local Optimization');
        
    tic;
    [output.theta_local,output.local_obj,output.local_exitflag]= nlopt_optimize(opt_local,mc.theta_0);
    output.time_local=toc;
    
    [~,output.mom_diff]=opt_local.min_objective(output.theta_local);
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    % Calculate the optimal weight matrix
    disp('Calculating Optimal Weight Matrix, Derivatives and Standard Errors');

    tic;
    [output.var_sim,output.D_local]=rc_D_V(output.theta_local,optim_set.theta_const,str_data,K,mom_data,sims_set.rng_seed+1000,sims_set.der_sims,optim_set.der_tol);            
    output.var_mom=rc_boot_markets(boot_set.var_boot,output.theta_local,str_data,K,mom_data);

    output.v_tot=output.var_mom+(1/sims_set.sims)*output.var_sim;  % Total Variance

    Sig1=output.D_local'*optim_set.W*output.D_local;        % Sig1 in sec 2.3 of Simulation Based Econometric Methods of Gourieroux and Monfort

    output.avar_theta_local=Sig1\(output.D_local'*optim_set.W*output.v_tot*optim_set.W*output.D_local)/Sig1;
    output.theta_local_se=sqrt(diag(output.avar_theta_local));
    output.time_se_local=toc;

    output.chi_sq_stat=output.mom_diff'*inv(output.v_tot)*output.mom_diff;
    output.reject=output.chi_sq_stat>chi2inv(0.95,size(output.mom_diff,1)-(K.b+K.g+K.int+K.rc));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Calculate a second stage estimate based on the optimal weight matrix
    disp('Calculating Second Stage Estimates');
    output.W_opt=inv(output.v_tot);
      
    opt_local.min_objective=@(theta) rc_msm_obj(theta,str_data,sim_data,K,mom_data,output.W_opt);

    tic;
    [output.theta_local_2,output.obj_local_2,output.exitflag_local_2]=nlopt_optimize(opt_local,output.theta_local);
    output.time_local_2=toc;

    [~,output.D_local_2]=rc_D_V(output.theta_local_2,optim_set.theta_const,str_data,K,mom_data,sims_set.rng_seed+100,sims_set.der_sims,optim_set.der_tol);            
    
    output.avar_theta_local_2=inv(output.D_local_2'*output.W_opt*output.D_local_2);
    output.theta_local_2_se=sqrt(diag(output.avar_theta_local_2));
    
    % Stats on second stage
    [~,output.mom_diff_2]=opt_local.min_objective(output.theta_local_2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Bootstrap, if desired
  
    if boot_set.boot_est
        tic;
        opt_boot=opt_local;
        opt_boot.lower_bounds=max([opt_local.lower_bounds output.theta_local_2-5*max(abs(output.theta_local_2),ones(size(output.theta_local_2)))],[],2);
        opt_boot.upper_bounds=min([opt_local.upper_bounds output.theta_local_2+5*max(abs(output.theta_local_2),ones(size(output.theta_local_2)))],[],2);

        [output.theta_boot,output.obj_boot,output.exitflag_boot]= rc_boot_est(boot_set.boot_est_iter,sims_set.sims,output.theta_local_2,opt_local,str_data,K,mom_data,output.W_opt);
        output.theta_local_2_boot_se= sqrt(diag(cov(output.theta_boot')));
        output.theta_local_2_ci=[prctile(output.theta_boot',5)' prctile(output.theta_boot',95)'];

        output.time_boot=toc;
    end
    

    
