/* Includes covariance formulations for within moments */

#include <mex.h>

/* Calculate the moments */

void calc_mom(double *match, double *X,double *Z,double *XZ,int N, int J, double *mom_ov, int mom_ov_K,double *mom_xz,int mom_xz_K,double *mom_w, int mom_w_K,double *mom_peer, int mom_peer_K,double *moments)
{
    int i,j,k;
    int m_x_k1_times_N,m_x_k2_times_N,m_z_k1_times_J,m_z_k2_times_J,m_xz_k_times_NJ,mom_loc;
    double one_on_N,x_z_1,x_z_2;
    double *x_z_sums,*num_match;      
    
    /* Store 1 on N */
    one_on_N=(double) 1/N;    
    
    /* Moments based on the overall match */
    for(k=0;k<mom_ov_K;k++) /* overall non XZ moments */
    {    
        /* Location of the column with the required variables     */
        m_x_k1_times_N=(int) (mom_ov[k*2]-1)*N;
        m_z_k1_times_J=(int) (mom_ov[k*2+1]-1)*J;
        for(i=0;i<N;i++)            
            moments[k]+=one_on_N*X[m_x_k1_times_N+((int) match[i])]*Z[m_z_k1_times_J+((int) match[N+i])];
    }
    for(k=0;k<mom_xz_K;k++)
    {
        mom_loc=k+mom_ov_K;
        m_xz_k_times_NJ=(int) (mom_xz[k]-1)*N*J;
	for(i=0;i<N;i++)
	    moments[mom_loc]+=one_on_N*XZ[m_xz_k_times_NJ+((int) match[i])*J+((int) match[N+i])];
    }
    

    
    /* Moments for within the match */
    x_z_sums=mxCalloc(J*mom_w_K,sizeof(double));
    num_match=mxCalloc(J,sizeof(double));
    for(k=0;k<mom_w_K;k++)
    {
        m_x_k1_times_N=(int) (mom_w[k*4]-1)*N;
        m_x_k2_times_N=(int) (mom_w[k*4+1]-1)*N;
	m_z_k1_times_J=(int) (mom_w[k*4+2]-1)*J;
	m_z_k2_times_J=(int) (mom_w[k*4+3]-1)*J;
	mom_loc=k+mom_ov_K+mom_xz_K;
	
	for(i=0;i<N;i++)
	{
	  if(m_x_k1_times_N!=m_x_k2_times_N || m_z_k1_times_J!=m_z_k2_times_J)
	    {
	      x_z_1=X[m_x_k1_times_N+((int) match[i])]*Z[m_z_k1_times_J+((int) match[N+i])]; /* This creates the sum of x_z's in the mom_w */
	      x_z_2=X[m_x_k2_times_N+((int) match[i])]*Z[m_z_k2_times_J+((int) match[N+i])]; /* This creates the sum of x_z's in the mom_w for the second one*/
	      x_z_sums[k*J+((int) match[N+i])]+=x_z_1;
	      moments[mom_loc]+=one_on_N*x_z_1*x_z_2;
	    }
	  else
	    {
	      x_z_1=X[m_x_k1_times_N+((int) match[i])]*Z[m_z_k1_times_J+((int) match[N+i])]; /* This creates the sum of x_z's in the mom_w */
	      x_z_sums[k*J+((int) match[N+i])]+=x_z_1;
	      moments[mom_loc]+=one_on_N*x_z_1*x_z_1;
	    }

	    if(k==0)
	      num_match[((int) match[N+i])]++;  /* The number matched needs to be counted only once */
	}

	for(j=0;j<J;j++)
	  if(num_match[j]>0)
	      moments[mom_loc]-=one_on_N*(1/num_match[j])*x_z_sums[k*J+j]*x_z_sums[k*J+j];
    } 

    /* Peer based moments */
    mxFree(x_z_sums);
    x_z_sums=mxCalloc(J*mom_peer_K,sizeof(double));

    /* Count the number of matches if this has not been done previously */
    if(mom_w_K==0)
      for(i=0;i<N;i++)
	num_match[(int) match[N+i]]++;


    for(k=0;k<mom_peer_K;k++)
    {
        m_x_k1_times_N=(int) (mom_peer[k*2]-1)*N;
	m_x_k2_times_N=(int) (mom_peer[k*2+1]-1)*N;

	mom_loc=k+mom_ov_K+mom_xz_K+mom_w_K;

	for(i=0;i<N;i++)
	    x_z_sums[k*J+((int) match[N+i])]+=X[m_x_k2_times_N+((int) match[i])];
	for(i=0;i<N;i++)
	  if(num_match[(int) match[N+i]]>1)
	    moments[mom_loc]+=one_on_N*(1/(num_match[(int) match[N+i]]-1))*X[m_x_k1_times_N+((int) match[i])]*(x_z_sums[k*J+((int) match[N+i])]-X[m_x_k2_times_N+((int) match[i])]);


    }
    
    /* Free Memory */
    mxFree(x_z_sums);
    mxFree(num_match);
}

    
    
