/* Takes a utiltiy vector on the verticals, 
 * a deterministic utility matrix on the horizontals,
 * simulation draws for the horizontals including the \beta_i rcs, to compute the match

 * Then, the moments are computed from a matrix of Xs, a matrix of Zs,  
 * and two sets of moment specifications (one within and one overall) 
 * to compute moments and matches. 
 * 
 * Returns only matches, if desired, or only matches and moments if desired */

#include <math.h>
#include <mex.h>
#include <matrix.h>

/* Function call [match, moments] = rc_moms_quick_c(u,v_det,eta_sim,beta_i_sim,Z_rc,qty,X,Z,XZ,mom_ov,mom_xz,mom_w,mom_peer) */

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int N,J,i,mom_ov_K,mom_w_K,mom_xz_K,mom_peer_K,K_rc;
          
    double *u,*v_det,*eta_sim,*beta_i_sim, *Z_rc, *qty, *X, *Z,*XZ, *mom_ov, *mom_w,*mom_xz,*mom_peer;
    double *match,*moments,*mom_covar;
        
    /* Get all the pointers */
    u=mxGetPr(prhs[0]);
    v_det=mxGetPr(prhs[1]);
    eta_sim=mxGetPr(prhs[2]);
    beta_i_sim=mxGetPr(prhs[3]);
    Z_rc=mxGetPr(prhs[4]);
    qty=mxGetPr(prhs[5]);

    N=mxGetM(prhs[0]);
    J=mxGetM(prhs[1]);
    K_rc=mxGetN(prhs[3]);
    mxAssert(K_rc==mxGetN(prhs[4]),"Mismatched number of Random Coeffs");   /* Check the number of RCs */

    if(nrhs>6)
    {    
        if(nrhs<13)
	   mexErrMsgTxt("Err: Not enough input arguments to mex File. Probably unspecified moments.");      

        X=mxGetPr(prhs[6]);
	Z=mxGetPr(prhs[7]);
	XZ=mxGetPr(prhs[8]);

	mom_ov=mxGetPr(prhs[9]);
	mom_xz=mxGetPr(prhs[10]);
	mom_w=mxGetPr(prhs[11]);
	mom_peer=mxGetPr(prhs[12]);

	mom_ov_K=mxGetN(prhs[9]);
	mom_xz_K=mxGetN(prhs[10]);
	mom_w_K=mxGetN(prhs[11]);  
	mom_peer_K=mxGetN(prhs[12]);    
   }
    
    /* Create a pointer to the outputs */
    plhs[0]=mxCreateDoubleMatrix(N,2,mxREAL);
    match=mxGetPr(plhs[0]);

    /* Do the calculations */
    calc_rc_match(match,u,v_det,beta_i_sim,Z_rc,eta_sim,qty,N,J,K_rc);
       
    /* Calculate moments */
    if(nlhs>1)
    {
        plhs[1]=mxCreateDoubleMatrix(mom_ov_K+mom_xz_K+mom_w_K+mom_peer_K,1,mxREAL);
        moments=mxGetPr(plhs[1]);            
	calc_mom(match,X,Z,XZ,N,J,mom_ov,mom_ov_K,mom_xz,mom_xz_K,mom_w,mom_w_K,mom_peer,mom_peer_K,moments);
    }

    /* Convert match to MATLAB indices */
    for(i=0;i<N;i++)
    {
        match[i]++;
        match[N+i]++;
    }
}
