function [str_data,K]=rc_init_struct_model(data,model)
    
    %First create the full matrix
    XZ_mat=kron(data.X,data.Z);
    XZ_names=cell(1,size(XZ_mat,2));
    
    % Append names
    for ii=1:size(data.X,2)
        for jj=1:size(data.Z,2)
            XZ_names{1,(ii-1)*size(data.Z,2)+jj}=[data.X_names{ii} '*' data.Z_names{jj}];
        end
    end
    
    % Append already prepared XZ Variables
    XZ_mat=[XZ_mat,data.XZ];
    XZ_names=cat(2,XZ_names,data.XZ_names);
    
    % Append constant
    data.X=[ones(size(data.X,1),1) data.X];
    data.Z=[ones(size(data.Z,1),1) data.Z];
    data.X_names={'const',data.X_names{:}};
    data.Z_names={'const',data.Z_names{:}};

    str_data.X_str=parse_names(data.X,data.X_names,model.u_X);
    if isfield(model,'u_het')
        str_data.X_het=parse_names(data.X,data.X_names,model.u_het);
    else
        str_data.X_het=zeros(data.N,0);
    end
    str_data.Z_str=parse_names(data.Z,data.Z_names,model.v_Z);
    str_data.XZ_str=parse_names(XZ_mat,XZ_names,model.v_XZ);
    
    str_data.X_rc=zeros(size(data.X,1),0);
    str_data.Z_rc=zeros(size(data.Z,1),0);
    
    if size(model.v_rc,2)>0
        str_data.X_rc=parse_names(data.X,data.X_names,model.v_rc(1,:));
        str_data.Z_rc=parse_names(data.Z,data.Z_names,model.v_rc(2,:));
    end
    
    % Other parameters
    str_data.N=data.N;
    str_data.J=data.J;
    str_data.qty=data.qty;
    str_data.p=str_data.N/sum(str_data.qty);
    
    % Number of Parameters
    if nargout>1
        K.b=size(str_data.X_str,2);
        K.het=size(str_data.X_het,2);
        K.g=size(str_data.Z_str,2);
        K.int=size(str_data.XZ_str,2);
        K.rc=size(str_data.Z_rc,2);
        K.K=K.b+K.het+K.g+K.int+K.rc;
    end
end

function [X_str]=parse_names(X,X_names,u_X)
    % Create X_str
    X_str=zeros(size(X,1),0);

    for ii=1:size(u_X,2)
        [is_mem,loc]=ismember(u_X(ii),X_names);
        if ~is_mem
            error(['Cannot find the variable: ' u_X{ii}]);
        end
        X_str=[X_str X(:,loc)];        
    end
end